@@ -1,6 +1,6 @@
 /*
  *  Methods for encryption/security mechanisms for cryptmount
- *  (C)Copyright 2007-2009, RW Penney
+ *  (C)Copyright 2007-2010, RW Penney
  */
 
 /*
@@ -501,7 +501,7 @@ keymanager_t keymgr_blti = {
                     kmblti_get_key, kmblti_put_key,
     &keymgr_pswd
 #ifdef TESTING
-    , NULL, NULL, 0
+    , NULL, NULL, CM_HASLEGACY
 #endif
 };
 
@@ -511,8 +511,6 @@ keymanager_t *kmblti_gethandle(void)
     return &keymgr_blti;
 }
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2007-2009, RW Penney
+ *  (C)Copyright 2007-2010, RW Penney
  */
