@@ -1,6 +1,6 @@
 /*
  *  Methods for encryption/security mechanisms for cryptmount
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
 
 /*
@@ -102,6 +102,7 @@ static void kmgcry_tx_algnames(const keyinfo_t *keyinfo,
         { "aes-256",    "aes256" },
         { "bf",         "blowfish" },
         { "cast",       "cast5" },
+        { "des3",       "3des" },
         { NULL, NULL }
     };
     struct map_t htable[] = {
@@ -383,9 +384,11 @@ static int kmgcry_initcipher(int cipher, int ciphermode, int digest,
 
 static int kmgcry_init_algs()
 {
-    (void)gcry_check_version(NULL);     /* Initializes library as side-effect */
+    if (!gcry_control(GCRYCTL_INITIALIZATION_FINISHED_P)) {
+        (void)gcry_check_version(NULL);     /* Initializes library as side-effect */
 
-    (void)gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+        (void)gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+    }
 
     return 0;
 }
@@ -832,7 +835,7 @@ keymanager_t keymgr_gcryossl = {
                       kmgcryossl_get_key, kmgcryossl_put_key,
     NULL
 #ifdef TESTING
-    , NULL, NULL, 0
+    , NULL, NULL, CM_HASLEGACY
 #endif
 };
 
@@ -848,7 +851,7 @@ keymanager_t keymgr_gcry = {
     NULL
 #endif
 #ifdef TESTING
-    , kmgcry_testctxt, kmgcry_runtests, 0
+    , kmgcry_testctxt, kmgcry_runtests, CM_HASLEGACY
 #endif
 };
 
@@ -879,8 +882,6 @@ keymanager_t *kmgcry_gethandle()
 
 #endif  /* !AS_MODULE */
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
