@@ -1,6 +1,6 @@
 /*
  *  methods for LUKS-related key-management for cryptmount
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
 
 /*
@@ -35,6 +35,7 @@
 
 #include "armour.h"
 #include "cryptmount.h"
+#include "dmutils.h"
 #include "looputils.h"
 #include "utils.h"
 #ifdef TESTING
@@ -53,6 +54,8 @@ typedef struct {
 
 #if USE_LUKSCOMPAT
 #  if !USE_MODULES || defined(AS_MODULE)
+#    include <gcrypt.h>
+#    include "luks/internal.h"
 #    include "luks/luks.h"
 #    if HAVE_UUID
 #      include <uuid/uuid.h>
@@ -118,8 +121,24 @@ static void kmluks_splitmode(const char *fullname, char **cipher, char **mode)
 
 
 static int kmluks_init_algs()
-{
-    /* Nothing needed */
+{   static int done_secmem = 0;
+    const char *GCRYPT_REQ_VERSION="1.1.42";
+
+    if (!done_secmem || !gcry_control(GCRYCTL_INITIALIZATION_FINISHED_P)) {
+        if (!gcry_check_version(GCRYPT_REQ_VERSION)) return -1;
+
+        /* Disable gcrypt secure-memory initialization as cryptmount makes
+         * its own arrangements for locking pages in memory.
+         * gcrypt secmem facilities will also drop setuid privileges,
+         * which would conflict with device-mapper system calls
+         * within cryptmount */
+		(void)gcry_control(GCRYCTL_DISABLE_SECMEM);
+
+        (void)gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+
+        done_secmem = 1;
+    }
+
     return 0;
 }
 
@@ -184,11 +203,12 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
             unsigned char **key, int *keylen, FILE *fp_key)
     /* Extract key from LUKS header file */
 {   tgtdefn_t *tgt = boundtgt->tgt;
-    char *passwd=NULL;
-    const char *tgtdev=NULL;
-    struct luks_phdr lukshdr;
-    struct luks_masterkey *lukskey=NULL;
-    int killloop=0, slot, eflag=ERR_NOERROR;
+    char *passwd = NULL;
+    const char *tgtdev = NULL;
+    struct crypt_device *luks_ctxt = NULL;
+    struct luks_phdr *lukshdr;
+    struct luks_masterkey *lukskey = NULL;
+    int killloop = 0, slot, eflag = ERR_NOERROR;
     luks_overrides_t *luksor;
     int64_t delta;
     const size_t namesz = LUKS_CIPHERNAME_L + LUKS_CIPHERMODE_L + 8;
@@ -202,17 +222,20 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
         goto bail_out;
     }
 
-    slot = LUKS_open_any_key(tgtdev, passwd, strlen(passwd),
-                    &lukshdr, &lukskey, &setup_libdevmapper_backend);
+    crypt_init(&luks_ctxt, tgtdev);
+    lukshdr = crypt_get_lukshdr(luks_ctxt);
+    LUKS_read_phdr(tgtdev, lukshdr, 1, luks_ctxt);
+    slot = LUKS_open_key_with_hdr(tgtdev, -1, passwd, strlen(passwd),
+                    lukshdr, &lukskey, luks_ctxt);
     if (slot < 0) {
-        fprintf(stderr, _("Failed to extract LUKS key for \"%s\"\n"),
-                tgt->ident);
+        fprintf(stderr, _("Failed to extract LUKS key for \"%s\" (errno=%d)\n"),
+                tgt->ident, -slot);
         eflag = ERR_BADDECRYPT;
         goto bail_out;
     }
 
     /* Extract cipher-algorithm parameter from LUKS header: */
-    delta = (lukshdr.payloadOffset - tgt->start);
+    delta = (lukshdr->payloadOffset - tgt->start);
     if (delta >= 0) {
         tgt->start += delta;
         if (tgt->length >= 0) tgt->length -= delta;
@@ -220,7 +243,7 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
     if (tgt->cipher != NULL) free((void*)tgt->cipher);
     tgt->cipher = (char*)malloc(namesz);
     snprintf(tgt->cipher, namesz, "%s-%s",
-            lukshdr.cipherName, lukshdr.cipherMode);
+            lukshdr->cipherName, lukshdr->cipherMode);
     tgt->ivoffset = 0;
     if (boundtgt->km_data != NULL) free((void*)boundtgt->km_data);
     luksor = (luks_overrides_t*)malloc(sizeof(luks_overrides_t));
@@ -235,8 +258,10 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
   bail_out:
 
     unblockify_file(&tgtdev, killloop);
+    udev_settle();
     if (passwd != NULL) sec_free((void*)passwd);
     if (lukskey != NULL) LUKS_dealloc_masterkey(lukskey);
+    if (luks_ctxt != NULL) crypt_free(luks_ctxt);
 
     return eflag;
 }
@@ -249,13 +274,15 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
 {
 #if HAVE_UUID
     const keyinfo_t *keyinfo = &boundtgt->tgt->key;
-    char *passwd=NULL, *ciphername=NULL, *ciphermode=NULL;
-    const char *tgtdev=NULL;
-    struct luks_phdr lukshdr;
-    struct luks_masterkey *lukskey=NULL;
-    unsigned keyslot=0;
-    luks_overrides_t *luksor=NULL;
-    int formatting=0, killloop=0, PBKDF2its, eflag=ERR_NOERROR;
+    char *passwd = NULL, *ciphername = NULL, *ciphermode = NULL;
+    const char *tgtdev = NULL;
+    struct crypt_device *luks_ctxt = NULL;
+    struct luks_phdr *lukshdr;
+    struct luks_masterkey *lukskey = NULL;
+    unsigned keyslot = 0;
+    uint64_t *iteration_time = NULL, *PBKDF2_per_sec = NULL;
+    luks_overrides_t *luksor = NULL;
+    int formatting = 0, killloop = 0, r, eflag = ERR_NOERROR;
 
     formatting = (boundtgt->km_data == NULL) && !kmluks_hdrvalid(fp_key);
 
@@ -283,16 +310,21 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
         goto bail_out;
     }
 
-    lukskey = LUKS_alloc_masterkey(keylen);
-    memcpy((void*)lukskey->key, (const void*)key, (size_t)keylen);
+    crypt_init(&luks_ctxt, tgtdev);
+    lukshdr = crypt_get_lukshdr(luks_ctxt);
+    crypt_get_pbkdf2params(luks_ctxt, &iteration_time, &PBKDF2_per_sec);
+    lukskey = LUKS_alloc_masterkey(keylen, (const char*)key);
 
     if (formatting) {
         /* Format new partition */
         keyslot = 0;
 
         kmluks_splitmode(boundtgt->tgt->cipher, &ciphername, &ciphermode);
-        if (LUKS_generate_phdr(&lukshdr, lukskey,
-                            ciphername, ciphermode, LUKS_STRIPES, 0) < 0) {
+        r = LUKS_generate_phdr(lukshdr, lukskey, ciphername, ciphermode,
+                            boundtgt->tgt->key.digestalg, NULL,
+                            LUKS_STRIPES, DEFAULT_ALIGNMENT,
+                            *iteration_time, PBKDF2_per_sec, luks_ctxt);
+        if (r < 0) {
             fprintf(stderr, _("Failed to create LUKS header for \"%s\"\n"),
                     boundtgt->tgt->ident);
             eflag = ERR_BADDEVICE;
@@ -302,21 +334,19 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
         /* Changing password on existing partition */
         keyslot = (luksor != NULL ? luksor->keyslot : 0);
 
-        if (LUKS_read_phdr(tgtdev, &lukshdr) < 0) {
+        if (LUKS_read_phdr(tgtdev, lukshdr, 1, luks_ctxt) < 0) {
             fprintf(stderr, _("Failed to read LUKS header for \"%s\"\n"),
                     boundtgt->tgt->ident);
             eflag = ERR_BADDEVICE;
             goto bail_out;
         }
 
-        lukshdr.keyblock[keyslot].active = LUKS_KEY_DISABLED;
+        lukshdr->keyblock[keyslot].active = LUKS_KEY_DISABLED;
     }
 
     printf(_("Setting password on LUKS keyslot-%u\n"), keyslot);
-    PBKDF2its = (int)(1.0 * LUKS_benchmarkt_iterations() + 0.5);
-    lukshdr.keyblock[keyslot].passwordIterations = (PBKDF2its >= 1 ? PBKDF2its : 1);
 
-    if (LUKS_set_key(tgtdev, keyslot, passwd, strlen(passwd), &lukshdr, lukskey, &setup_libdevmapper_backend) < 0) {
+    if (LUKS_set_key(tgtdev, keyslot, passwd, strlen(passwd), lukshdr, lukskey, *iteration_time, PBKDF2_per_sec, luks_ctxt) < 0) {
         fprintf(stderr, _("Failed to create LUKS key for \"%s\"\n"),
                 boundtgt->tgt->ident);
         fprintf(stderr, "LUKS error: %s\n", get_error());
@@ -327,10 +357,12 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
   bail_out:
 
     unblockify_file(&tgtdev, killloop);
+    udev_settle();
     if (lukskey != NULL) LUKS_dealloc_masterkey(lukskey);
     if (passwd != NULL) sec_free((void*)passwd);
     if (ciphername != NULL) free((void*)ciphername);
     if (ciphermode != NULL) free((void*)ciphermode);
+    if (luks_ctxt != NULL) crypt_free(luks_ctxt);
 
     return eflag;
 
@@ -396,7 +428,7 @@ keymanager_t keymgr_luks = {
                       kmluks_get_key, kmluks_put_key,
     NULL
 #ifdef TESTING
-    , kmluks_testctxt, kmluks_runtests, (CM_READONLY)
+    , kmluks_testctxt, kmluks_runtests, (CM_READONLY | CM_HASLEGACY)
 #endif
 };
 
@@ -427,8 +459,6 @@ keymanager_t *kmluks_gethandle()
 
 #endif  /* !AS_MODULE */
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
