@@ -1,6 +1,6 @@
 /*
  *  Methods for encryption/security mechanisms for cryptmount
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
 
 /*
@@ -186,7 +186,7 @@ static keymanager_t keymgr_raw = {
                     kmraw_get_key, kmraw_put_key,
     NULL
 #ifdef TESTING
-    , NULL, NULL, 0
+    , NULL, NULL, CM_HASLEGACY
 #endif
 };
 
@@ -256,9 +256,9 @@ const char **get_keymgr_list(void)
 keymanager_t *init_keymanager(keymanager_t *km)
 {
     if (km != NULL) {
-        if (!km->initialized) {
+        if ((km->initialized & KM_INIT_ALGS) == 0) {
             if (km->init_algs() == 0) {
-                km->initialized = 1;
+                km->initialized |= KM_INIT_ALGS;
             } else {
                 fprintf(stderr, "Failed to initialize keymanager \"%s\"\n",
                         km->ident);
@@ -274,7 +274,7 @@ int free_keymanagers(void)
 {   keymanager_t *km;
 
     for (km=keymgrs; km!=NULL; km=km->next) {
-        if (km->initialized) {
+        if ((km->initialized & KM_INIT_ALGS) != 0) {
             km->free_algs();
             km->initialized = 0;
         }
@@ -566,7 +566,8 @@ int km_archive_keys(void)
 
 int km_test_legacy(void)
     /* Check that keyfiles from earlier releases can be read */
-{   DIR *dp;
+{   keymanager_t *km;
+    DIR *dp;
     const struct dirent *dent;
     struct stat sbuff;
     tgtdefn_t *target;
@@ -585,6 +586,10 @@ int km_test_legacy(void)
     dp = opendir(dirpath);
     if (dp == NULL) CM_TEST_FAIL();
 
+    for (km=keymgrs; km!=NULL; km=km->next) {
+        km->initialized &= ~KM_TESTED;
+    }
+
     for (;;) {
         dent = readdir(dp);
         if (dent == NULL) break;
@@ -598,13 +603,18 @@ int km_test_legacy(void)
         (void)strtok(tokbuff, "_");
 
         target = alloc_tgtdefn(NULL);
+        target->ident = cm_strdup(dent->d_name);
         target->key.format = cm_strdup(strtok(NULL, "_"));
         target->key.filename = cm_strdup(keypath);
         target->key.digestalg = cm_strdup(strtok(NULL, "_"));
         target->key.cipheralg = cm_strdup(strtok(NULL, "_"));
 
         bound = bind_tgtdefn(target);
-        if (bound == NULL) CM_TEST_FAIL();
+        if (bound != NULL) {
+            ((keymanager_t*)bound->keymgr)->initialized |= KM_TESTED;
+        } else {
+            continue;   /* No available key manager - possibly not compiled? */
+        }
 
         key = NULL;
         keylen = 0;
@@ -621,9 +631,6 @@ int km_test_legacy(void)
         } else {
             CM_TEST_FAIL();
         }
-#if 0
-        fprintf(stderr, "%s -> %s  %d\n", keypath, bound->keymgr->ident, flg);
-#endif
 
   skip_jail:
 
@@ -637,6 +644,14 @@ int km_test_legacy(void)
 
     if (n_legacy < 4) CM_TEST_FAIL();
 
+    /* Check that all available keymanagers with legacy keys have been tested */
+    for (km=keymgrs; km!=NULL; km=km->next) {
+        if ((km->test_flags & CM_HASLEGACY) != 0
+          && (km->initialized & KM_TESTED) == 0) {
+            CM_TEST_FAIL();
+        }
+    }
+
     CM_TEST_OK();
 }
 
@@ -759,7 +774,7 @@ int sycheck_target(const tgtdefn_t *tgt)
 
 
 /*
- *  ==== mutex-locking on configuration directory ====
+ *  ==== Mutex-locking on configuration directory ====
  */
 
 int cm_mutex_lock(void)
@@ -826,8 +841,6 @@ int cm_mutex_unlock(void)
     return eflag;
 }
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
