@@ -1,6 +1,6 @@
 /*
  *  Declarations for encryption/security mechanisms for cryptmount
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
 
 /*
@@ -37,7 +37,7 @@ struct km_overrides;
 typedef struct keymanager {
     const char *ident;
 
-    int initialized;
+    unsigned initialized;
     int (*init_algs)(void);
     int (*free_algs)(void);
 
@@ -68,6 +68,12 @@ typedef struct keymanager {
 #endif
 } keymanager_t;
 
+/* Key-manager initialization status flags: */
+enum {
+    KM_INIT_ALGS =          0x001,
+    KM_TESTED =             0x800
+};
+
 /* Key-manager key-properties flags: */
 enum {
     KM_PROP_HASPASSWD =     0x001,      /* Password needed to access key */
@@ -128,8 +134,6 @@ int cm_mutex_unlock(void);
 
 #endif  /* _ARMOUR_H */
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
