@@ -1,6 +1,6 @@
 /*
  *  Methods for unit-testing utiltities for cryptmount
- *  (C)Copyright 2006-2009, RW Penney
+ *  (C)Copyright 2006-2010, RW Penney
  */
 
 /*
@@ -73,12 +73,13 @@ int cm_tests_close()
 
 
 int cm_run_tests()
-    /* front-end to self-testing routines */
+    /* Front-end to self-testing routines */
 {
     cm_tests_init();
 
     ut_test_strops();
     fs_test_blkgetsz();
+    fs_test_splitopts();
     ut_test_sha1();
     ut_pwfort();
     bf_test_blowfish();
@@ -99,5 +100,5 @@ int _keep_ansi_pedantic_quiet = 0;
 /* vim: set ts=4 sw=4 et: */
 
 /*
- *  (C)Copyright 2006-2009, RW Penney
+ *  (C)Copyright 2006-2010, RW Penney
  */
