@@ -1,6 +1,6 @@
 /*
  *  Declarations for unit-test utilities for cryptmoumt
- *  (C)Copyright 2006-2009, RW Penney
+ *  (C)Copyright 2006-2010, RW Penney
  */
 
 /*
@@ -36,7 +36,8 @@ enum {
 };
 
 enum {
-    CM_READONLY = 1
+    CM_READONLY =   0x001,              /* Omit tests of key-creation */
+    CM_HASLEGACY =  0x002               /* Test against legacy key files */
 };
 
 
@@ -113,8 +114,6 @@ int cm_run_tests();
 
 #endif  /* _CMTEST_H */
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2006-2009, RW Penney
+ *  (C)Copyright 2006-2010, RW Penney
  */
