@@ -1,6 +1,6 @@
 /*
  *  General declarations for cryptmount
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
 
 /*
@@ -130,7 +130,8 @@ typedef struct tgtdefn      /* Entry in list of cryptmount filesystem-targets */
     int64_t start, length;  /* Starting sector + num of sectors (or 0, -1) */
     char *dir;              /* Mount-point */
     char *fstype;           /* Filesystem type */
-    char *fsoptions;        /* Mount options */
+    char *mountoptions;     /* Options passed to 'mount' command */
+    char *fsckoptions;      /* Options passed to 'fsck' command */
     char *loopdev;          /* Loopback device to wrap around raw file */
 
     char *cipher;           /* Cipher used on filesystem */
@@ -143,8 +144,6 @@ typedef struct tgtdefn      /* Entry in list of cryptmount filesystem-targets */
 
 #endif  /* _CRYPTMOUNT_H */
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
