@@ -1,6 +1,6 @@
 /*
- *  device-mapper utilities for cryptmount
- *  (C)Copyright 2005-2009, RW Penney
+ *  Device-mapper utilities for cryptmount
+ *  (C)Copyright 2005-2010, RW Penney
  */
 
 /*
@@ -45,6 +45,10 @@
 #ifndef UDEV_QUEUE_DIR
 #  define UDEV_QUEUE_DIR "/dev/.udev/queue"
 #endif
+#ifndef UDEV_QUEUE_FILE
+#  define UDEV_QUEUE_FILE "/dev/.udev/queue.bin"
+#endif
+int udev_queue_size();
 
 
 struct dm_task *devmap_prepare(int type, const char *ident)
@@ -233,12 +237,13 @@ int is_configured(const char *ident, struct dm_info *dminfo)
 int udev_settle()
     /* Allow time for udev events to be processed */
 {   double totdelay = 0.0, inc;
+    time_t starttime;
     struct stat sbuff;
 #if HAVE_NANOSLEEP
     struct timespec delay;
 #endif
-    int outstanding = 1;
-    const double timeout = 3.0;
+    int settling;
+    const double timeout = 5.0;
 
     /* This routine mitigates apparent race-conditions
      * between udev events which may temporarily take ownership of
@@ -249,6 +254,8 @@ int udev_settle()
      * to resolve, or for user applications to mitigate,
      * is open to debate. */
 
+    time(&starttime);
+
 #if HAVE_NANOSLEEP
     delay.tv_sec = 0.0;
     delay.tv_nsec = 2e7;
@@ -257,6 +264,7 @@ int udev_settle()
     inc = 1.0;
 #endif
 
+    /* Keep waiting until there are no more queued udev events: */
     do {
 #if HAVE_NANOSLEEP
         nanosleep(&delay, NULL);
@@ -265,18 +273,50 @@ int udev_settle()
 #endif
         totdelay += inc;
 
-        /* Keep waiting if there are any queued udev events: */
-        if (stat(UDEV_QUEUE_DIR, &sbuff) < 0) {
-            outstanding = 0;
-            break;
-        }
-    } while (totdelay < timeout);
+        settling = 0;
 
-    return outstanding;
+        /* Older versions of udev place events in separate directory */
+        if (stat(UDEV_QUEUE_DIR, &sbuff) == 0) {
+            settling |= ((starttime - sbuff.st_mtime) < 100 * timeout);
+        }
+
+        /* Newer versions of udev place events in a queue file */
+        settling |= (udev_queue_size() > 0);
+    } while (settling && totdelay < timeout);
+
+    return settling;
 }
 
-/* vim: set ts=4 sw=4 et: */
+
+int udev_queue_size()
+    /* Count number of unprocessed udev events in queue.bin file */
+{   FILE *fp;
+    unsigned long long seqnum;
+    unsigned short skiplen;
+    int nqueued = 0;
+
+    fp = fopen(UDEV_QUEUE_FILE, "rb");
+    if (fp == NULL) return 0;
+    if (fread((void*)&seqnum, sizeof(seqnum), (size_t)1, fp) != 1) return 0;
+
+    for (;;) {
+        skiplen = 0;
+        if (fread((void*)&seqnum, sizeof(seqnum), (size_t)1, fp) != 1
+          || fread((void*)&skiplen, sizeof(skiplen), (size_t)1, fp) != 1) break;
+
+        if (skiplen > 0) {
+            void *buff = malloc((size_t)skiplen);
+            nqueued += (fread(buff, (size_t)skiplen, (size_t)1, fp) == 1);
+            free(buff);
+        } else {
+            --nqueued;
+        }
+    }
+    fclose(fp);
+
+    return nqueued;
+}
 
 /*
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
