@@ -1,6 +1,6 @@
 /*
  *  Config-table and mount-table utilities for cryptmount
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
 
 /*
@@ -83,7 +83,8 @@ tgtdefn_t *alloc_tgtdefn(const tgtdefn_t *prototype)
     tgt->start = 0; tgt->length = -1;
     tgt->dir = NULL;
     tgt->fstype = DFLT_OR_PROTO(NULL, cm_strdup(prototype->fstype));
-    tgt->fsoptions = DFLT_OR_PROTO(NULL, cm_strdup(prototype->fsoptions));
+    tgt->mountoptions = DFLT_OR_PROTO(NULL, cm_strdup(prototype->mountoptions));
+    tgt->fsckoptions = DFLT_OR_PROTO(NULL, cm_strdup(prototype->fsckoptions));
     tgt->loopdev = NULL;
 
     tgt->cipher = DFLT_OR_PROTO(cm_strdup(DFLT_CIPHER),
@@ -135,7 +136,8 @@ tgtdefn_t *clone_tgtdefn(const tgtdefn_t *orig)
     clone->length = orig->length;
     clone->dir = cm_strdup(orig->dir);
     clone->fstype = cm_strdup(orig->fstype);
-    clone->fsoptions = cm_strdup(orig->fsoptions);
+    clone->mountoptions = cm_strdup(orig->mountoptions);
+    clone->fsckoptions = cm_strdup(orig->fsckoptions);
     clone->loopdev = cm_strdup(orig->loopdev);
 
     clone->cipher = cm_strdup(orig->cipher);
@@ -162,7 +164,8 @@ void free_tgtdefn(tgtdefn_t *tgt)
     if (tgt->dev != NULL) free((void*)tgt->dev);
     if (tgt->dir != NULL) free((void*)tgt->dir);
     if (tgt->fstype != NULL) free((void*)tgt->fstype);
-    if (tgt->fsoptions != NULL) free((void*)tgt->fsoptions);
+    if (tgt->mountoptions != NULL) free((void*)tgt->mountoptions);
+    if (tgt->fsckoptions != NULL) free((void*)tgt->fsckoptions);
     if (tgt->loopdev != NULL) free((void*)tgt->loopdev);
 
     if (tgt->cipher != NULL) free((void*)tgt->cipher);
@@ -275,7 +278,9 @@ static void read_token(char *buff, unsigned *t_state, tgtdefn_t *tgt)
         { "startsector",    OFFSET(start),          proc_int64 },
         { "numsectors",     OFFSET(length),         proc_int64 },
         { "fstype",         OFFSET(fstype),         proc_string },
-        { "fsoptions",      OFFSET(fsoptions),      proc_string },
+        { "mountoptions",   OFFSET(mountoptions),   proc_string },
+        { "fsoptions",      OFFSET(mountoptions),   proc_string },  /* Deprecated */
+        { "fsckoptions",    OFFSET(fsckoptions),    proc_string },
         { "loop",           OFFSET(loopdev),        proc_string },
         { "cipher",         OFFSET(cipher),         proc_string },
         { "ivoffset",       OFFSET(ivoffset),       proc_int64 },
@@ -334,9 +339,9 @@ static void read_token(char *buff, unsigned *t_state, tgtdefn_t *tgt)
 tgtdefn_t *parse_stream(FILE *fp, const char *cfgname)
     /* Convert config-file information into linked-list of target-structures */
 {   enum { C_SPACE, C_WORD, C_COMMENT };
-    int ch,lineno=1,literal=0;
-    unsigned pos=0,bufflen=0;
-    unsigned c_state,t_state;
+    int ch, lineno=1, literal=0;
+    unsigned pos=0, bufflen=0;
+    unsigned c_state, t_state;
     char *buff=NULL;
     tgtdefn_t *head=NULL, **cmp=&head, *tgt=NULL, *prototype=NULL;
 
@@ -684,5 +689,5 @@ int put_tgtstatus(const tgtdefn_t *tgt, const tgtstat_t *newstat)
 /* vim: set ts=4 sw=4 et: */
 
 /*
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
