@@ -1,6 +1,6 @@
 /*
  *  miscellaneous utilitie functions for cryptmount
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
 
 /*
@@ -243,8 +243,10 @@ int get_randkey(unsigned char *buff, size_t len)
         /* Fold-in entropy from random-number devices: */
         for (i=0; devs[i].name!=NULL; ++i) {
             if (devs[i].fp == NULL) continue;
-            (void)fread((void*)devbuff, (size_t)1, (size_t)blklen, devs[i].fp);
-            cm_sha1_block(mdcontext, devbuff, blklen);
+            sz = fread((void*)devbuff, (size_t)1, (size_t)blklen, devs[i].fp);
+            if (sz > 0) {
+                cm_sha1_block(mdcontext, devbuff, blklen);
+            }
         }
 
         /* Fold-in existing bytes from key: */
@@ -299,7 +301,7 @@ ssize_t cm_ttygetpasswd(const char *prompt, char **buff)
     if (tcgetattr(fileno(stdin), &newttystate) != 0
       || (newttystate.c_lflag & ECHO) != 0) echook = 0;
     if (echook) {
-        printf(prompt);
+        printf("%s", prompt);
         if (fgets(tmppass, (int)sizeof(tmppass), stdin) == NULL) {
             fprintf(stderr, _("Cannot read stdin"));
             return -1;
@@ -843,8 +845,6 @@ int ut_pwfort()
 
 #endif  /* TESTING */
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2005-2009, RW Penney
+ *  (C)Copyright 2005-2010, RW Penney
  */
